//--------------------------------------------------------------
// File     : xmc2go_ub_adc.c
// Datum    : 19.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : XMC1100-0064 (XMC2GO)
// IDE      : KEIL uVision 5.10
// ARMCC    : 5.04
// Module   : XMC1100
// Funktion : ADC-Funktionen
//
// Hinweise : ADC1 = P2.6
//--------------------------------------------------------------


/*******************************************************************************
 Copyright (c) 2013, Infineon Technologies AG                                 **
 All rights reserved.                                                         **
                                                                              **
 Redistribution and use in source and binary forms, with or without           **
 modification,are permitted provided that the following conditions are met:   **
                                                                              **
 *Redistributions of source code must retain the above copyright notice,      **
 this list of conditions and the following disclaimer.                        **
 *Redistributions in binary form must reproduce the above copyright notice,   **
 this list of conditions and the following disclaimer in the documentation    **
 and/or other materials provided with the distribution.                       **
 *Neither the name of the copyright holders nor the names of its contributors **
 may be used to endorse or promote products derived from this software without**
 specific prior written permission.                                           **
                                                                              **
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
 ARE  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   **
 LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         **
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
 SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    **
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
 CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       **
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   **
 POSSIBILITY OF SUCH DAMAGE.                                                  **
                                                                              **
 To improve the quality of the software, users are encouraged to share        **
 modifications, enhancements or bug fixes with Infineon Technologies AG       **
 dave@infineon.com).                                                          **
*******************************************************************************/





//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "xmc2go_ub_adc.h"


//--------------------------------------------------------------
// init vom ADC
//--------------------------------------------------------------
void XMC2GO_Adc_Init(void)
{
  SCU_GENERAL->PASSWD = 0x000000C0;
  SCU_CLK->CGATCLR0 |= 0x00000001;	// disable VADC gating
  while((SCU_CLK->CLKCR)&0x40000000);	// wait for VDDC to stabilize
  SCU_GENERAL->PASSWD = 0x000000C3;

  // Initialize adc on P2.6
  PORT2->IOCR4 &= ~PORT2_IOCR4_PC6_Msk;                                 // Set P2.6 as Input.
  VADC->CLC &= ~VADC_CLC_DISR_Msk;                                      // Enable Clock for the analog section.
  SHS0->SHSCFG = (SHS_SHSCFG_SCWC_Msk);                                 // Enable write access to ANOFF and AREF
  SHS0->SHSCFG &= ~(SHS_SHSCFG_ANOFF_Msk);                              // Toggle on converter.
  *((int*)0x40010500) = 0x01;                                           // workaround to enable converter.
  // SHS0->SHSCFG  TODO: set AREF
  while ((SHS0->SHSCFG & SHS_SHSCFG_ANRDY_Msk) == 0);                   // Wait until converter is ready.

  // VADC->GLOBCFG &= ~VADC_GLOBCFG_DPCAL0_Msk;                         // Enable post-calibration.
  // workaround for calibration
  VADC->GLOBCFG |= VADC_GLOBCFG_DPCAL0_Msk;
  VADC->GLOBCFG |= VADC_GLOBCFG_DPCAL1_Msk;
  VADC->GLOBCFG &= ~VADC_GLOBCFG_SUCAL_Msk;                             // Raising edge on SUCAL to initiate the calibration.
  VADC->GLOBCFG |= VADC_GLOBCFG_SUCAL_Msk;
  // workaround for calibration
  VADC->GLOBCFG |= VADC_GLOBCFG_DPCAL0_Msk;
  VADC->GLOBCFG |= VADC_GLOBCFG_DPCAL1_Msk;
  while ((SHS0->SHSCFG & SHS_SHSCFG_STATE_Msk) != 0);                   // Wait until calibration is finished.
  *((int*)0x480340E0) = 0x80008000;
  *((int*)0x480340E4) = 0x80008000;

  VADC->BRSMR |= 0x01UL;                                                // Gate always enabled.
  VADC->BRSSEL[0] |= VADC_BRSSEL_CHSELG0_Msk;                           // Select P2.6 as input channel.
  VADC->GLOBICLASS[0] |= (0x01UL << VADC_GLOBICLASS_CMS_Pos);           // Select 10-Bit conversion for channel 0.	
}


//--------------------------------------------------------------
// ADC auslesen
// ret_wert : [0...1023]
//--------------------------------------------------------------
uint16_t XMC2GO_Adc_Read(uint8_t adc_nr)
{
  uint16_t ret_wert=0;
	
  if(adc_nr==0) {
    VADC->BRSPND[0] |= VADC_BRSPND_CHPNDG0_Msk;                     // Set P2.6 to pending channel to start conversion.
    while ((VADC->GLOBRES & VADC_GLOBRES_VF_Msk) == 0);             // Wait until valid flag is set.
    ret_wert = (VADC->GLOBRES & VADC_GLOBRES_RESULT_Msk) >> 2;      // Save the result.
	
    if(ret_wert>MAX_ADC_WERT) ret_wert=MAX_ADC_WERT;
  }
	
  return(ret_wert);
}

