//--------------------------------------------------------------
// File     : xmc2go_ub_gpio.c
// Datum    : 16.05.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : XMC1100-0064 (XMC2GO)
// IDE      : KEIL uVision 5.10
// ARMCC    : 5.04
// Module   : GPIO
// Funktion : Digital IN/OUT
//
// Hinweise : Input-0  = P2.10
//            Input-1  = P0.0
//            Input-2  = P2.9
//            Input-3  = P2.11
//            Input-4  = P2.7
//
//            Output-0 = P0.5
//            Output-1 = P2.0
//--------------------------------------------------------------


/*******************************************************************************
 Copyright (c) 2013, Infineon Technologies AG                                 **
 All rights reserved.                                                         **
                                                                              **
 Redistribution and use in source and binary forms, with or without           **
 modification,are permitted provided that the following conditions are met:   **
                                                                              **
 *Redistributions of source code must retain the above copyright notice,      **
 this list of conditions and the following disclaimer.                        **
 *Redistributions in binary form must reproduce the above copyright notice,   **
 this list of conditions and the following disclaimer in the documentation    **
 and/or other materials provided with the distribution.                       **
 *Neither the name of the copyright holders nor the names of its contributors **
 may be used to endorse or promote products derived from this software without**
 specific prior written permission.                                           **
                                                                              **
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
 ARE  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   **
 LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         **
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
 SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    **
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
 CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       **
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   **
 POSSIBILITY OF SUCH DAMAGE.                                                  **
                                                                              **
 To improve the quality of the software, users are encouraged to share        **
 modifications, enhancements or bug fixes with Infineon Technologies AG       **
 dave@infineon.com).                                                          **
*******************************************************************************/



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "xmc2go_ub_gpio.h"



//--------------------------------------------------------------
// Init der Digital Pins
//--------------------------------------------------------------
void XMC2GO_GPIO_Init(void)
{
  //-----------------------------------------
  // GPIO config (input mit Pullup)
  //
  // Hinweis : 
  //    P2.0 bis P2.11 muessen auf Digital geschaltet werden
  //    Bsp : P2_6_enable_digital();
  //-----------------------------------------
	
  // IN0=P2.10
  P2_10_enable_digital();
  P2_10_set_mode(INPUT_PU);	    
  // IN1=P0.0
  P0_0_set_mode(INPUT_PU);
  // IN2=P2.9
  P2_9_enable_digital();
  P2_9_set_mode(INPUT_PU);
  // IN3=P2.11
  P2_11_enable_digital();
  P2_11_set_mode(INPUT_PU);	
  // IN4=P2.7
  P2_7_enable_digital();
  P2_7_set_mode(INPUT_PU);  


  //-----------------------------------------
  // GPIO config (pushpull output)
  //
  // Hinweis : 
  //    P2.0 bis P2.11 muessen auf Digital geschaltet werden
  //    Bsp : P2_6_enable_digital();
  //-----------------------------------------

  // OUT0=P0.5
  P0_5_set_mode(OUTPUT_PP_GP);
  // OUT1=P2.0
  P2_0_enable_digital();
  P2_0_set_mode(OUTPUT_PP_GP);  
	
  // default werte einstellen
  P0_5_reset();
  P2_0_reset();
}

//--------------------------------------------------------------
// auslesen eines Digital-In Pins
// ret_wert : [0...1] 0=Lo, 1=Hi
//--------------------------------------------------------------
uint8_t XMC2GO_DIn_Read(uint8_t din_name)
{
  uint8_t ret_wert=0;
  uint16_t wert=0;

  if(din_name==DIN0) {
    wert=P2_10_read();
    if(wert!=0) ret_wert=1;
  }	
  else if(din_name==DIN1) {
    wert=P0_0_read();
    if(wert!=0) ret_wert=1;
  }		
  else if(din_name==DIN2) {
    wert=P2_9_read();
    if(wert!=0) ret_wert=1;
  }
  else if(din_name==DIN3) {
    wert=P2_11_read();
    if(wert!=0) ret_wert=1;
  }
  else if(din_name==DIN4) {
    wert=P2_7_read();
    if(wert!=0) ret_wert=1;
  }	

  return(ret_wert);
}

//--------------------------------------------------------------
// Pin auf Lo-Pegel schalten
//--------------------------------------------------------------
void XMC2GO_DOut_Lo(uint8_t dout_name)
{
  if(dout_name==DOUT0) {
    P0_5_reset();
  }
  else if(dout_name==DOUT1) {
    P2_0_reset();
  }  
}

//--------------------------------------------------------------
// Pin auf Hi-Pegel schalten
//--------------------------------------------------------------
void XMC2GO_DOut_Hi(uint8_t dout_name)
{
  if(dout_name==DOUT0) {
    P0_5_set();
  }
  else if(dout_name==DOUT1) {
    P2_0_set();
  }  
}

//--------------------------------------------------------------
// Pin-Pegel toggeln
//--------------------------------------------------------------
void XMC2GO_DOut_Toggle(uint8_t dout_name)
{
  if(dout_name==DOUT0) {
    P0_5_toggle();
  }
  else if(dout_name==DOUT1) {
    P2_0_toggle();
  }  
}



