//--------------------------------------------------------------
// File     : xmc2go_ub_nokia5110.h
//--------------------------------------------------------------

//--------------------------------------------------------------
#ifndef __XMC2GO_UB_NOKIA5110_H
#define __XMC2GO_UB_NOKIA5110_H


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include <stdint.h>
#include "gpio.h"
#include "xmc2go_ub_spi.h"




//--------------------------------------------------------------
// Defines vom Display
//--------------------------------------------------------------
#define  LCD_MAXX           ((uint16_t)84)      // Pixel in X-Richtung
#define  LCD_MAXY           ((uint16_t)48)      // Pixel in Y-Richtung
#define  LCD_PIXEL  LCD_MAXX*LCD_MAXY

#define  LCD_LINES           LCD_MAXY/8         // Anzahl der Zeilen
#define  LCD_BYTES  LCD_MAXX*LCD_LINES          // Anzahl der Bytes



//--------------------------------------------------------------
// sonstige defines
//--------------------------------------------------------------
#define  LCD_INIT_PAUSE   500000  // kleine Pause beim Init




//--------------------------------------------------------------
// Struktur von einem Font (max 16 Pixel Breite)
//-------------------------------------------------------------- 
typedef struct UB_Font_t {
  const uint16_t *table; // Tabelle mit den Daten
  uint16_t width;        // Breite eines Zeichens (in Pixel)
  uint16_t height;       // Hoehe eines Zeichens  (in Pixel)
}UB_Font;


//--------------------------------------------------------------
// Aktivierung der verschiedenen Fonts
// (unbenutzte einfach auskommentieren)
// fuer alle benutzten Fonts muss das entsprechende C-File
// in der CooCox-IDE hinzugefgt werden
//--------------------------------------------------------------
extern UB_Font Font_6x8;



//--------------------------------------------------------------
// Globale Funktionen
//--------------------------------------------------------------
void XMC2GO_LCD_Init(void);
uint8_t XMC2GO_LCD_Refresh(void);
void XMC2GO_LCD_Inverse(uint8_t mode);
void XMC2GO_LCD_Clear(void);
void XMC2GO_LCD_SetPixel(uint8_t xpos, uint8_t ypos);
void XMC2GO_LCD_ClearPixel(uint8_t xpos, uint8_t ypos);
uint8_t XMC2GO_LCD_ReadPixel(uint8_t xpos, uint8_t ypos);
void XMC2GO_LCD_DrawLine(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void XMC2GO_LCD_DrawRect(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void XMC2GO_LCD_DrawFill(uint8_t x1, uint8_t y1, uint8_t x2, uint8_t y2);
void XMC2GO_LCD_DrawCircle(uint8_t x0, uint8_t y0, uint8_t radius);
void XMC2GO_LCD_DrawChar(uint8_t x, uint8_t y, uint8_t ascii, UB_Font *font);
void XMC2GO_LCD_DrawString(uint8_t x, uint8_t y,char *ptr, UB_Font *font);
void XMC2GO_LCD_ClearCollision(void);
uint8_t XMC2GO_LCD_GetCollision(void);


//--------------------------------------------------------------
#endif // __XMC2GO_UB_NOKIA5110_H
