//--------------------------------------------------------------
// File     : xmc2go_ub_spi.c
// Datum    : 16.05.2014
// Version  : 1.1 (angepasst fuer uPlay)
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : XMC1100-0064 (XMC2GO)
// IDE      : KEIL uVision 5.10
// ARMCC    : 5.04
// Module   : XMC1100, GPIO
// Funktion : SPI-Funktionen
//
// Hinweise : MISO = P0.6 (wird benutzt fuer Break-Button)
//            MOSI = P0.7
//            CLK  = P0.8
//            SlaveSel = P0.9
//--------------------------------------------------------------


/*******************************************************************************
 Copyright (c) 2013, Infineon Technologies AG                                 **
 All rights reserved.                                                         **
                                                                              **
 Redistribution and use in source and binary forms, with or without           **
 modification,are permitted provided that the following conditions are met:   **
                                                                              **
 *Redistributions of source code must retain the above copyright notice,      **
 this list of conditions and the following disclaimer.                        **
 *Redistributions in binary form must reproduce the above copyright notice,   **
 this list of conditions and the following disclaimer in the documentation    **
 and/or other materials provided with the distribution.                       **
 *Neither the name of the copyright holders nor the names of its contributors **
 may be used to endorse or promote products derived from this software without**
 specific prior written permission.                                           **
                                                                              **
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
 ARE  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   **
 LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         **
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
 SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    **
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
 CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       **
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   **
 POSSIBILITY OF SUCH DAMAGE.                                                  **
                                                                              **
 To improve the quality of the software, users are encouraged to share        **
 modifications, enhancements or bug fixes with Infineon Technologies AG       **
 dave@infineon.com).                                                          **
*******************************************************************************/


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "xmc2go_ub_spi.h"



//--------------------------------------------------------------
uint8_t spi_rec_wert;


//--------------------------------------------------------------
// Init vom SPI
//--------------------------------------------------------------
void XMC2GO_Spi_Init(void)
{
  spi_rec_wert=0xFF;
	
  P0_6_set_mode(INPUT_PU);			//USIC0_CH1.DX0C

	
  NVIC_ClearPendingIRQ(USIC0_0_IRQn);		//This function clears interrupt node pending status
  NVIC_EnableIRQ(USIC0_0_IRQn);			//This function enables interrupt node for interrupt request generation
  	
	
  //----USIC-SETUP------------------------------------------------------------------------------------
  SCU_GENERAL->PASSWD = 0x000000C0UL;
  SCU_CLK->CGATCLR0 |= 0x00000008;		// stop gating USIC
  while((SCU_CLK->CLKCR)&0x40000000UL);	// wait for VDDC to stabilize
  SCU_GENERAL->PASSWD = 0x000000C3UL;

  //Kernel State Configuration Register - Module Enable + Bit Protection for MODEN
  USIC0_CH1->KSCFG |= (1 << USIC_CH_KSCFG_MODEN_Pos) | (1 << USIC_CH_KSCFG_BPMODEN_Pos);

  //Normal divider mode selected
  WR_REG(USIC0_CH1->FDR, USIC_CH_FDR_DM_Msk, USIC_CH_FDR_DM_Pos, 1);
  //Configuration of baud rate
  WR_REG(USIC0_CH1->FDR, USIC_CH_FDR_STEP_Msk, USIC_CH_FDR_STEP_Pos, SPI_FDR_STEP);
  WR_REG(USIC0_CH1->BRG, USIC_CH_BRG_PDIV_Msk, USIC_CH_BRG_PDIV_Pos, SPI_BRG_PDIV);
	
  #if SPI_MODE==0 || SPI_MODE==1
    WR_REG(USIC0_CH1->BRG, USIC_CH_BRG_SCLKCFG_Msk, USIC_CH_BRG_SCLKCFG_Pos, 0); // CPOL=0
  #endif
  #if SPI_MODE==2 || SPI_MODE==3
    WR_REG(USIC0_CH1->BRG, USIC_CH_BRG_SCLKCFG_Msk, USIC_CH_BRG_SCLKCFG_Pos, 1); // CPOL=1
  #endif	
	
  //Configuration of USIC Input Stage
  WR_REG(USIC0_CH1->DX0CR, USIC_CH_DX0CR_DSEL_Msk, USIC_CH_DX0CR_DSEL_Pos, 2);
  WR_REG(USIC0_CH1->DX0CR, USIC_CH_DX0CR_INSW_Msk, USIC_CH_DX0CR_INSW_Pos, 1);
  WR_REG(USIC0_CH1->DX2CR, USIC_CH_DX2CR_DSEL_Msk, USIC_CH_DX2CR_DSEL_Pos, 7);

  //Configuration of USIC Shift Control
  //Transmit/Receive MSB first is selected, Transmission Mode (TRM) = 1, Passive Data Level (PDL) = 1
  WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_PDL_Msk, USIC_CH_SCTR_PDL_Pos, 1);
  WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_TRM_Msk, USIC_CH_SCTR_TRM_Pos, 1);

  #if SPI_BITORDER==0
    WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_SDIR_Msk, USIC_CH_SCTR_SDIR_Pos, 1);  // MSB First
  #endif
  #if SPI_BITORDER==1
    WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_SDIR_Msk, USIC_CH_SCTR_SDIR_Pos, 0);  // LSB First
  #endif
  //Set Word Length (WLE) & Frame Length (FLE)
  WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_FLE_Msk, USIC_CH_SCTR_FLE_Pos, 7); // 7=8bit Data pro Frame
  WR_REG(USIC0_CH1->SCTR, USIC_CH_SCTR_WLE_Msk, USIC_CH_SCTR_WLE_Pos, 7); // 7=8bit Data pro Frame

  //Configuration of USIC Transmit Control/Status Register
  //TBUF Data Enable (TDEN) = 1, TBUF Data Single Shot Mode (TDSSM) = 1
  WR_REG(USIC0_CH1->TCSR, USIC_CH_TCSR_TDEN_Msk, USIC_CH_TCSR_TDEN_Pos, 1);
  WR_REG(USIC0_CH1->TCSR, USIC_CH_TCSR_TDSSM_Msk, USIC_CH_TCSR_TDSSM_Pos, 1);

  //Configuration of Protocol Control Register
  WR_REG(USIC0_CH1->PCR_SSCMode, USIC_CH_PCR_SSCMode_MSLSEN_Msk, USIC_CH_PCR_SSCMode_MSLSEN_Pos, 1);
  WR_REG(USIC0_CH1->PCR_SSCMode, USIC_CH_PCR_SSCMode_SELCTR_Msk, USIC_CH_PCR_SSCMode_SELCTR_Pos, 1);
  WR_REG(USIC0_CH1->PCR_SSCMode, USIC_CH_PCR_SSCMode_SELINV_Msk, USIC_CH_PCR_SSCMode_SELINV_Pos, 1);
  WR_REG(USIC0_CH1->PCR_SSCMode, USIC_CH_PCR_SSCMode_SELO_Msk, USIC_CH_PCR_SSCMode_SELO_Pos, 1);

  //Configuration of Channel Control Register
  WR_REG(USIC0_CH1->CCR, USIC_CH_CCR_MODE_Msk, USIC_CH_CCR_MODE_Pos, 1);
  WR_REG(USIC0_CH1->CCR, USIC_CH_CCR_AIEN_Msk, USIC_CH_CCR_AIEN_Pos, 1);
	
  P0_9_set_mode(OUTPUT_PP_AF7);	//USIC0_CH1.SELO0
  P0_7_set_mode(OUTPUT_PP_AF7);	//USIC0_CH1.DOUT0
  P0_8_set_mode(OUTPUT_PP_AF7);	//USIC0_CH1.SCLKOUT	
}


//--------------------------------------------------------------
// sendet und empfngt ein Byte per SPI
//--------------------------------------------------------------
uint8_t XMC2GO_Spi_SendByte(uint8_t wert)
{
  uint8_t ret_wert=0;
	
  USIC0_CH1->TBUF[0]=(uint8_t)(wert);
	
  ret_wert=spi_rec_wert;
	
  return(ret_wert);
}


//--------------------------------------------------------------
// ISR vom SPI
//--------------------------------------------------------------
void USIC0_0_IRQHandler(void)
{	
  if((RD_REG(USIC0_CH1->PSR_SSCMode, USIC_CH_PSR_SSCMode_AIF_Msk, USIC_CH_PSR_SSCMode_AIF_Pos))==1)
  {
    SET_BIT(USIC0_CH1->PSCR, USIC_CH_PSCR_CAIF_Pos);
    spi_rec_wert = (uint8_t)((RD_REG(USIC0_CH1->RBUF, USIC_CH_RBUF_DSR_Msk, USIC_CH_RBUF_DSR_Pos)));
  }
}

