//--------------------------------------------------------------
// File     : xmc2go_ub_uPlay.c
// Datum    : 16.05.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : XMC1100-0064 (XMC2GO)
// IDE      : KEIL uVision 5.10
// ARMCC    : 5.04
// Module   : XMC2GO_NOKIA5110, XMC2GO_UART, XMC2GO_GPIO
// Funktion : uPlay
//--------------------------------------------------------------



//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "xmc2go_ub_uPlay.h"
#include <stdio.h>
#include <string.h>
#include "bas_snake.h"


//--------------------------------------------------------------
// interne Funktionen
//--------------------------------------------------------------
void p_uPlay_ShowMenu(uint8_t mode);
void p_uPlay_CopyFlash(void);
void p_uPlay_Run(void);
void p_uPlay_Load(void);
void p_uPlay_SendInfo(void);
void p_uPlay_ShowInfo(void);


//--------------------------------------------------------------
// Speicher fuer Basic-Programm im RAM
//--------------------------------------------------------------
char ram_program_01[UPLAY_RAM_SIZE];


//--------------------------------------------------------------
// Globale Variabeln
//--------------------------------------------------------------
char rx_buf[RX_BUF_SIZE];
char str_buf[15];	
UART_RXSTATUS_t rx_status;



//--------------------------------------------------------------
// init vom uPlay-System
//--------------------------------------------------------------
void XMC2GO_uPlay_Init(void)
{
  uint32_t n;

  // init vom Display
  XMC2GO_LCD_Init();
  // init der UART
  XMC2GO_Uart_Init();
  // init der GPIOs
  XMC2GO_GPIO_Init();
  // init vom ADC
  XMC2GO_Adc_Init();

  for(n=0;n<UPLAY_RAM_SIZE;n++) {
    ram_program_01[n]=0x00;
  }	
}

//--------------------------------------------------------------
// start vom uPlay-System (endlos)
//--------------------------------------------------------------
void XMC2GO_uPlay_Start(void)
{
  uint8_t key=0;
  uint8_t mode=1, status=1;
	
  p_uPlay_SendInfo();
  p_uPlay_ShowInfo();
  
  p_uPlay_ShowMenu(0);
	
  while(1) {	
	
    p_uPlay_ShowMenu(mode);
    if(status==1) {
      status=0;
      XMC2GO_Uart_SendString("[Standby-Mode]",CRLF);
    }
	
    // warte auf Tastendruck (oder UART Command);
    do {
      key=0;
      if(XMC2GO_DIn_Read(DIN2)==0) key=1; // left
      if(XMC2GO_DIn_Read(DIN3)==0) key=2; // right
      if(XMC2GO_DIn_Read(DIN4)==0) key=3; // enter
			
      rx_status=XMC2GO_Uart_ReceiveString(rx_buf);
      if(rx_status==RX_READY) {
        if(strcmp(rx_buf,UPLAY_RUN_CMD)==0) {
          key=4;
        }	
        else if(strcmp(rx_buf,UPLAY_LOAD_CMD)==0) {
          key=5;
        }
        else if(strcmp(rx_buf,UPLAY_HELP_CMD)==0) {
          p_uPlay_SendInfo();
        }
      }
    }
    while(key==0);
	
    if(key==1) mode=1;  
    if(key==2) mode=2;
    if(key==3) {
      if(mode==1) p_uPlay_CopyFlash();
      p_uPlay_Run();
      p_uPlay_ShowMenu(0);
      status=1; 
    }
    if(key==4) {
      p_uPlay_Run();
      p_uPlay_ShowMenu(0);
      status=1;
    }
    if(key==5) {
      p_uPlay_Load();
      p_uPlay_ShowMenu(0);
      status=1;
    }
		
    // warte bis keine Taste gedrueckt
    do {
      key=0;
      if(XMC2GO_DIn_Read(DIN2)==0) key=1; // left
      if(XMC2GO_DIn_Read(DIN3)==0) key=2; // right
      if(XMC2GO_DIn_Read(DIN4)==0) key=3; // enter
    }
    while(key!=0);
  }
}

//--------------------------------------------------------------
// interne Funktion
// Anzeige vom Menu (je nach Mode)
//--------------------------------------------------------------
void p_uPlay_ShowMenu(uint8_t mode)
{
  if(mode==0) {
    XMC2GO_LCD_Clear();
    XMC2GO_LCD_DrawString(0,0 ,"XMC2Go-uPlay",&Font_6x8);
    XMC2GO_LCD_DrawLine(0,10,83,10);
    XMC2GO_LCD_DrawString(0,15,">FLASH<   RAM ",&Font_6x8);
    XMC2GO_LCD_DrawString(0,35," [L/R/ENTER] ",&Font_6x8);
  }    
  else if(mode==1) {
    XMC2GO_LCD_DrawString(0,15,">FLASH<   RAM ",&Font_6x8);  
  }
  else if(mode==2) {
    XMC2GO_LCD_DrawString(0,15," FLASH   >RAM<",&Font_6x8);
  }  
}

//--------------------------------------------------------------
// interne Funktion
// kopiert ein Basic-Programm aus dem Flash ins RAM
//--------------------------------------------------------------
void p_uPlay_CopyFlash(void)
{
  uint32_t n;
  char wert;
	
  // flash in RAM kopieren
  n=0;
  do {
    wert=flash_game_snake[n];
    ram_program_01[n]=wert;
    n++;
  }
  while(wert!=0x00);

  ram_program_01[n]=0x00;	
}

//--------------------------------------------------------------
// interne Funktion
// startet ein Basic-Programm aus dem RAM
//--------------------------------------------------------------
void p_uPlay_Run(void)
{
  uint8_t break_btn=0;
	
  XMC2GO_LCD_Clear();
  XMC2GO_LCD_DrawString(0,0 ,"pre-parse...",&Font_6x8);	  
  XMC2GO_Uart_SendString("pre-parse...",CRLF);
	
  ubasic_init(ram_program_01);
  if(ubasic_failed()) {    
    sprintf(str_buf,"error:%d",ubasic_failed());
    XMC2GO_LCD_DrawString(0,10,str_buf,&Font_6x8);
    XMC2GO_Uart_SendString(str_buf,CRLF);
    sprintf(str_buf,"in line:%d",ubasic_linenum());
    XMC2GO_LCD_DrawString(0,20,str_buf,&Font_6x8);
    XMC2GO_Uart_SendString(str_buf,CRLF);	
    XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_RUN_ERR);
    return;
  }

  XMC2GO_Uart_SendString("[Run-Mode]",CRLF);
  XMC2GO_LCD_DrawString(0,10,"OK",&Font_6x8);
  XMC2GO_LCD_DrawString(0,20,"run prg...",&Font_6x8);
	
  XMC2GO_Systic_enableLED2(1);
  do {
    ubasic_run();
    break_btn=XMC2GO_Systic_checkBreakBtn();
  } while(!ubasic_finished() && break_btn==0xFF);
  XMC2GO_Systic_enableLED2(0);
		
  XMC2GO_LCD_Clear();
  XMC2GO_LCD_DrawString(0,0 ,"pre-parse...",&Font_6x8);	 
  XMC2GO_LCD_DrawString(0,10,"OK",&Font_6x8);
  XMC2GO_LCD_DrawString(0,20,"run prg...",&Font_6x8);
		
  if(break_btn!=0xFF) {
    XMC2GO_LCD_DrawString(0,30,"break",&Font_6x8);
    XMC2GO_Uart_SendString("break by user",CRLF);
    XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_RUN_OK);    
  }
  else {
    if(!ubasic_failed()) {
      XMC2GO_LCD_DrawString(0,30,"ready",&Font_6x8);
      XMC2GO_Uart_SendString("ready",CRLF);
      XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_RUN_OK);
    }
    else {				
      sprintf(str_buf,"error:%d",ubasic_failed());
      XMC2GO_LCD_DrawString(0,30,str_buf,&Font_6x8);
      XMC2GO_Uart_SendString(str_buf,CRLF);
      sprintf(str_buf,"in line:%d",ubasic_linenum());
      XMC2GO_LCD_DrawString(0,40,str_buf,&Font_6x8);
      XMC2GO_Uart_SendString(str_buf,CRLF);
      XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_RUN_ERR);
    }
  }				  
}

//--------------------------------------------------------------
// interne Funktion
// empfangen von einem Basic-Programm per UART
// und speichern im RAM
//--------------------------------------------------------------
void p_uPlay_Load(void)
{
  uint8_t ok=0;
  uint32_t line_cnt=0,n,l,ptr=0;
	
  XMC2GO_LCD_Clear();
  XMC2GO_LCD_DrawString(0,0 ,"loading...",&Font_6x8);
  XMC2GO_LCD_DrawLine(0,10,83,10);
  XMC2GO_Uart_SendString("[Load-Mode]",CRLF);	
	
  XMC2GO_LCD_DrawString(0,15 ,"lines=0",&Font_6x8);
	
  line_cnt=0;
  ok=0;
  ptr=0;
  do {
    rx_status=XMC2GO_Uart_ReceiveString(rx_buf);
    if(rx_status==RX_READY) {
      if(strcmp(rx_buf,UPLAY_END_CMD)==0) {
        ok=1;
      }
      else {
        line_cnt++;
        sprintf(str_buf,"lines=%d",line_cnt);
        XMC2GO_LCD_DrawString(0,15 ,str_buf,&Font_6x8);
        l=strlen(rx_buf);
        for(n=0;n<l;n++) {
          if(ptr<UPLAY_RAM_SIZE) {
            ram_program_01[ptr]=rx_buf[n];
            ptr++;
          }
          else {
            ok=2;
          }
        }
        if(ptr<UPLAY_RAM_SIZE) {
          ram_program_01[ptr]='\n';
          ptr++;
        }
        else {
          ok=2;
        }
      }
    }
  }while(ok==0);
	
  if(ok==1) {
    ram_program_01[ptr]=0x00;
    XMC2GO_LCD_DrawString(0,25 ,"receive OK",&Font_6x8);
    sprintf(str_buf,"receive ok, bytes=%d",ptr);
    XMC2GO_Uart_SendString(str_buf,CRLF);
  }
  else {
    XMC2GO_LCD_DrawString(0,25 ,"out of memory",&Font_6x8);
    sprintf(str_buf,"memory overflow, bytes=%d",ptr);
    XMC2GO_Uart_SendString(str_buf,CRLF);	
  }
	
  XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_LOAD);
}

//--------------------------------------------------------------
// interne Funktion
// senden der Versions-Daten per UART
//--------------------------------------------------------------
void p_uPlay_SendInfo(void)
{
  XMC2GO_Systic_Pause_ms(5);
  XMC2GO_Uart_SendString("XMC2Go uPlay",CRLF);
  XMC2GO_Systic_Pause_ms(5);
  sprintf(str_buf,"uPlay Ver.:%s",UPLAY_VER);
  XMC2GO_Uart_SendString(str_buf,CRLF);
  XMC2GO_Systic_Pause_ms(5);
  sprintf(str_buf,"uBasic Ver.:%s",UBASIC_VER);
  XMC2GO_Uart_SendString(str_buf,CRLF);	
}

//--------------------------------------------------------------
// interne Funktion
// anzeigen der Versions-Daten auf dem Display
//--------------------------------------------------------------
void p_uPlay_ShowInfo(void)
{
  XMC2GO_LCD_Clear();
  XMC2GO_LCD_DrawString(0,0 ,"XMC2Go-uPlay",&Font_6x8);
  XMC2GO_LCD_DrawLine(0,10,83,10);  
  sprintf(str_buf,"uPlay  :%s",UPLAY_VER);
  XMC2GO_LCD_DrawString(0,15,str_buf,&Font_6x8);
  sprintf(str_buf,"uBasic :%s",UBASIC_VER);
  XMC2GO_LCD_DrawString(0,25,str_buf,&Font_6x8);

  XMC2GO_Systic_Pause_ms(UPLAY_PAUSE_INFO);  
}
