//--------------------------------------------------------------
// File     : xmc2go_ub_uart.c
// Datum    : 19.04.2014
// Version  : 1.0
// Autor    : UB
// EMail    : mc-4u(@)t-online.de
// Web      : www.mikrocontroller-4u.de
// CPU      : XMC1100-0064 (XMC2GO)
// IDE      : KEIL uVision 5.10
// ARMCC    : 5.04
// Module   : XMC1100, GPIO
// Funktion : UART
//
// Hinweise : TX = P2.1
//            RX = P2.2
//            Setting = 115200Baud/8N1
//
// Vorsicht : Als Endekennung beim Empfangen, muss der Sender
//            das Zeichen "0x0D" = Carriage-Return
//            an den String anhngen !!
//--------------------------------------------------------------


/*******************************************************************************
 Copyright (c) 2013, Infineon Technologies AG                                 **
 All rights reserved.                                                         **
                                                                              **
 Redistribution and use in source and binary forms, with or without           **
 modification,are permitted provided that the following conditions are met:   **
                                                                              **
 *Redistributions of source code must retain the above copyright notice,      **
 this list of conditions and the following disclaimer.                        **
 *Redistributions in binary form must reproduce the above copyright notice,   **
 this list of conditions and the following disclaimer in the documentation    **
 and/or other materials provided with the distribution.                       **
 *Neither the name of the copyright holders nor the names of its contributors **
 may be used to endorse or promote products derived from this software without**
 specific prior written permission.                                           **
                                                                              **
 THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"  **
 AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE    **
 IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE   **
 ARE  DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE   **
 LIABLE  FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR         **
 CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF         **
 SUBSTITUTE GOODS OR  SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS    **
 INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN      **
 CONTRACT, STRICT LIABILITY,OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)       **
 ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE   **
 POSSIBILITY OF SUCH DAMAGE.                                                  **
                                                                              **
 To improve the quality of the software, users are encouraged to share        **
 modifications, enhancements or bug fixes with Infineon Technologies AG       **
 dave@infineon.com).                                                          **
*******************************************************************************/


//--------------------------------------------------------------
// Includes
//--------------------------------------------------------------
#include "xmc2go_ub_uart.h"


//--------------------------------------------------------------
UART_RX_t XMC2GO_RX;


//--------------------------------------------------------------
// init der UART
//--------------------------------------------------------------
void XMC2GO_Uart_Init(void)
{
  uint8_t dummy,n;
	
  XMC2GO_RX.status=RX_EMPTY;
  XMC2GO_RX.wr_ptr=0;
  XMC2GO_RX.rx_buffer[0]=RX_END_CHR;
	
  // auf Digital schalten
  P2_1_enable_digital();
  P2_2_enable_digital();	
	
  // Disable clock gating to USIC0
  SCU_GENERAL->PASSWD = 0x000000C0UL; // disable bit protection
  SCU_CLK->CGATCLR0 |= SCU_CLK_CGATCLR0_USIC0_Msk;
  SCU_GENERAL->PASSWD = 0x000000C3UL; // enable bit protection

  /* Enable the module kernel clock and the module functionality  */
  USIC0_CH0->KSCFG |= USIC_CH_KSCFG_MODEN_Msk | USIC_CH_KSCFG_BPMODEN_Msk;

  /* fFD = fPB */
  /* FDR.DM = 02b (Fractional divider mode) */
  USIC0_CH0->FDR &= ~(USIC_CH_FDR_DM_Msk | USIC_CH_FDR_STEP_Msk);
  USIC0_CH0->FDR |= (0x02UL << USIC_CH_FDR_DM_Pos) | (UART_FDR_STEP << USIC_CH_FDR_STEP_Pos);

  /* Configure baud rate generator */
  /* BAUDRATE = fCTQIN/(BRG.PCTQ x BRG.DCTQ) */
  /* CLKSEL = 0 (fPIN = fFD), CTQSEL = 00b (fCTQIN = fPDIV), PPPEN = 0 (fPPP=fPIN) */
  USIC0_CH0->BRG &= ~(USIC_CH_BRG_PCTQ_Msk | USIC_CH_BRG_DCTQ_Msk | USIC_CH_BRG_PDIV_Msk | USIC_CH_BRG_CLKSEL_Msk | USIC_CH_BRG_PPPEN_Msk);
  USIC0_CH0->BRG |= (UART_BRG_PCTQ << USIC_CH_BRG_PCTQ_Pos) | (UART_BRG_DCTQ << USIC_CH_BRG_DCTQ_Pos) | (UART_BRG_PDIV << USIC_CH_BRG_PDIV_Pos);

  /* Configuration of USIC Shift Control */
  /* SCTR.FLE = 8 (Frame Length) */
  /* SCTR.WLE = 8 (Word Length) */
  /* SCTR.TRM = 1 (Transmission Mode) */
  /* SCTR.PDL = 1 (This bit defines the output level at the shift data output signal when no data is available for transmission) */
  USIC0_CH0->SCTR &= ~(USIC_CH_SCTR_TRM_Msk | USIC_CH_SCTR_FLE_Msk | USIC_CH_SCTR_WLE_Msk);
  USIC0_CH0->SCTR |= USIC_CH_SCTR_PDL_Msk | (0x01UL << USIC_CH_SCTR_TRM_Pos) | (0x07UL << USIC_CH_SCTR_FLE_Pos) | (0x07UL << USIC_CH_SCTR_WLE_Pos);

  /* Configuration of USIC Transmit Control/Status Register */
  /* TBUF.TDEN = 1 (TBUF Data Enable: A transmission of the data word in TBUF can be started if TDV = 1 */
  /* TBUF.TDSSM = 1 (Data Single Shot Mode: allow word-by-word data transmission which avoid sending the same data several times*/
  USIC0_CH0->TCSR &= ~(USIC_CH_TCSR_TDEN_Msk);
  USIC0_CH0->TCSR |= USIC_CH_TCSR_TDSSM_Msk | (0x01UL << USIC_CH_TCSR_TDEN_Pos);

  /* Configuration of Protocol Control Register */
  /* PCR.SMD = 1 (Sample Mode based on majority) */
  /* PCR.STPB = 0 (1x Stop bit) */
  /* PCR.SP = 5 (Sample Point) */
  /* PCR.PL = 0 (Pulse Length is equal to the bit length) */
  USIC0_CH0->PCR &= ~(USIC_CH_PCR_ASCMode_STPB_Msk | USIC_CH_PCR_ASCMode_SP_Msk | USIC_CH_PCR_ASCMode_PL_Msk);
  USIC0_CH0->PCR |= USIC_CH_PCR_ASCMode_SMD_Msk | (9 << USIC_CH_PCR_ASCMode_SP_Pos);

  /* Configure Transmit Buffer */
  /* Standard transmit buffer event is enabled */
  /* Define start entry of Transmit Data FIFO buffer DPTR = 0*/
  USIC0_CH0->TBCTR &= ~(USIC_CH_TBCTR_SIZE_Msk | USIC_CH_TBCTR_DPTR_Msk);

  /* Set Transmit Data Buffer to 32 and set data pointer to position 0*/
  USIC0_CH0->TBCTR |= ((0x05 << USIC_CH_TBCTR_SIZE_Pos)|(0x00 << USIC_CH_TBCTR_DPTR_Pos));

  /* Initialize UART_RX (DX0)*/
  /* P2.2 as input */
  P2_2_set_mode(INPUT);

  /* Select P2.2 as input for USIC DX3 -> UCIC DX0 */
  USIC0_CH0->DX3CR &= ~(USIC_CH_DX3CR_DSEL_Msk);

  /* Route USIC DX3 input signal to USIC DX0 (DSEL=DX0G) */
  USIC0_CH0->DX0CR &= ~(USIC_CH_DX0CR_DSEL_Msk);
  USIC0_CH0->DX0CR |= 6 << USIC_CH_DX0CR_DSEL_Pos;

  /* Configure Receive Buffer */
  /* Standard Receive buffer event is enabled */
  /* Define start entry of Receive Data FIFO buffer DPTR = 32*/
  USIC0_CH0->RBCTR &= ~(USIC_CH_RBCTR_SIZE_Msk | USIC_CH_RBCTR_DPTR_Msk);

  /* Set Receive Data Buffer Size to 32 and set data pointer to position 32*/
  USIC0_CH0->RBCTR |= ((0x05UL << USIC_CH_RBCTR_SIZE_Pos)|(32 << USIC_CH_RBCTR_DPTR_Pos));

  /* Initialize UART_TX (DOUT)*/
  /* P2.1 as output controlled by ALT6 = U0C0.DOUT0 */
  P2_1_set_mode(OUTPUT_PP_AF6);

  /* Configuration of Channel Control Register */
  /* CCR.PM = 00 ( Disable parity generation) */
  /* CCR.MODE = 2 (ASC mode enabled. Note: 0 (USIC channel is disabled)) */
  USIC0_CH0->CCR &= ~(USIC_CH_CCR_PM_Msk | USIC_CH_CCR_MODE_Msk);
  USIC0_CH0->CCR |= 0x02UL << USIC_CH_CCR_MODE_Pos;	
	
  // RX-Puffer leeren
  for(n=0;n<100;n++) {
    if ((USIC0_CH0->TRBSR & 0x00000008) == 0){
      dummy=(USIC0_CH0->OUTR & 0x0000FFFF);
      USIC0_CH0->IN[0]=dummy;		 
    }
  }
}


//--------------------------------------------------------------
// ein Byte per UART senden
//--------------------------------------------------------------
void XMC2GO_Uart_SendByte(uint8_t wert)
{
  USIC0_CH0->IN[0] = (uint8_t)(wert);
}


//--------------------------------------------------------------
// einen String per UART senden
//--------------------------------------------------------------
void XMC2GO_Uart_SendString(char *ptr, UART_LASTBYTE_t end_cmd)
{
  // sende kompletten String
  while (*ptr != 0) {
    XMC2GO_Uart_SendByte(*ptr);
    ptr++;
  }
  // eventuell Endekennung senden
  if(end_cmd==LFCR) {
    XMC2GO_Uart_SendByte(0x0A); // LineFeed senden
    XMC2GO_Uart_SendByte(0x0D); // CariageReturn senden
  }
  else if(end_cmd==CRLF) {    
    XMC2GO_Uart_SendByte(0x0D); // CariageReturn senden
    XMC2GO_Uart_SendByte(0x0A); // LineFeed senden
  }
  else if(end_cmd==LF) {    
    XMC2GO_Uart_SendByte(0x0A); // LineFeed senden
  }
  else if(end_cmd==CR) {    
    XMC2GO_Uart_SendByte(0x0D); // CariageReturn senden    
  }
}


//--------------------------------------------------------------
// einen String per UART empfangen
// diese Funktion muss zyklisch gepollt werden
// Return Wert :
//  -> wenn nichts empfangen = RX_EMPTY
//  -> wenn String empfangen = RX_READY -> String steht in *ptr
//  -> wenn Puffer voll      = RX_FULL
//--------------------------------------------------------------
UART_RXSTATUS_t XMC2GO_Uart_ReceiveString(char *ptr)
{
  UART_RXSTATUS_t ret_wert=RX_EMPTY;
  uint8_t wert;
  uint32_t n;

  if ((USIC0_CH0->TRBSR & 0x00000008) == 0){
    wert=(uint8_t)((USIC0_CH0->OUTR & 0x0000FFFF));
 
    if(XMC2GO_RX.wr_ptr<RX_BUF_SIZE) {
      // wenn noch Platz im Puffer
      if(XMC2GO_RX.status==RX_EMPTY) {
        // wenn noch keine Endekennung empfangen wurde
        if((wert>=RX_FIRST_CHR) && (wert<=RX_LAST_CHR)) {
          // Byte im Puffer speichern
          XMC2GO_RX.rx_buffer[XMC2GO_RX.wr_ptr]=wert;
          XMC2GO_RX.wr_ptr++;
        }
        if(wert==RX_END_CHR) {
          // wenn Endekennung empfangen
          XMC2GO_RX.rx_buffer[XMC2GO_RX.wr_ptr]=wert;
          XMC2GO_RX.status=RX_READY;
        }
      }
    }
    else {
      // wenn Puffer voll ist
      ret_wert=RX_FULL;
      XMC2GO_RX.wr_ptr=0;
      XMC2GO_RX.status=RX_EMPTY;
      XMC2GO_RX.rx_buffer[0]=RX_END_CHR;
    }
  }
	
  if(XMC2GO_RX.status==RX_READY) {
    ret_wert=RX_READY;
    n=0;
    do {
      wert=XMC2GO_RX.rx_buffer[n];
      if(wert!=RX_END_CHR) {
        ptr[n]=wert;
        n++;
      }
    }while(wert!=RX_END_CHR);
    // Stringendekennung
    ptr[n]=0x00;
    // RX-Puffer lschen
    XMC2GO_RX.rx_buffer[0]=RX_END_CHR;
    XMC2GO_RX.wr_ptr=0;
    XMC2GO_RX.status=RX_EMPTY;		
  }

  return(ret_wert);
}

